use actix_web::web::ServiceConfig;
use chrono::NaiveDate;
use serde::Deserialize;

mod events;
mod location;
mod user;
mod assignment;
mod get_export;

#[derive(Deserialize)]
pub struct IdPath {
    pub id: i32,
}

#[derive(Deserialize)]
pub struct NaiveDateQuery {
    pub date: NaiveDate,
}

pub fn init(cfg: &mut ServiceConfig) {
    cfg.service(location::get_overview::get);
    cfg.service(location::get_new::get);
    cfg.service(location::post_new::post);

    cfg.service(user::get_overview::get_overview);
    cfg.service(user::get_new::get_new);
    cfg.service(user::post_new::post_new);
    cfg.service(user::get_edit::get_edit);
    cfg.service(user::post_edit::post_edit);
    cfg.service(user::patch::patch);
    cfg.service(user::delete::delete);
    cfg.service(user::get_logout::get);
    cfg.service(user::get_login::get);
    cfg.service(user::post_login::post);
    cfg.service(user::get_reset::get);
    cfg.service(user::post_reset::post);

    cfg.service(events::get_new::get);
    cfg.service(events::post_new::post);

    cfg.service(assignment::get_new::get);
    cfg.service(assignment::post_new::post);

    cfg.service(get_export::get);
}
