use actix_identity::Identity;
use actix_web::{web, HttpResponse, Responder};
use serde::Deserialize;
use serde_json::value::Value;
use sqlx::PgPool;

use crate::{
    endpoints::IdPath,
    models::{Role, User},
};

#[derive(Deserialize)]
pub struct JsonPatchDoc {
    op: String,
    path: String,
    value: Value
}

#[actix_web::patch("/users/edit/{id}")]
pub async fn patch(
    user: Identity,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
    patch_docs: web::Json<Vec<JsonPatchDoc>>,
) -> impl Responder {
    let current_user = User::read_by_id(pool.get_ref(), user.id().unwrap().parse().unwrap())
        .await
        .unwrap();

    if current_user.role != Role::AreaManager && current_user.role != Role::Admin {
        return HttpResponse::Unauthorized().finish();
    }

    if let Ok(user_in_db) = User::read_by_id(pool.get_ref(), path.id).await {
        if current_user.role == Role::AreaManager && current_user.area_id != user_in_db.area_id {
            return HttpResponse::Unauthorized().finish();
        }

        let mut changed = false;

        let mut locked: Option<bool> = None;

        for doc in patch_docs.iter() {
            if doc.op.as_str() != "replace" {
                continue;
            }

            match doc.path.as_str() {
                "/locked" => {
                    changed = true;
                    if let Value::Bool(b) = doc.value {
                        locked = Some(b);
                    }
                },
                _ => panic!("other patch paths are not supported!"),
            };
        }

        if changed {
            if let Ok(_) = User::update(pool.get_ref(), path.id, None, None, None, None, None, None, None, locked).await {
                return HttpResponse::Ok().body("");
            }
        } else {
                return HttpResponse::Ok().body("");
        }
    }

    HttpResponse::BadRequest().body("Fehler bei User PATCH")
}
